@@ -98,17 +98,37 @@ int cudaDeviceGetAttributeFromC(int *value, int attribute) {
   }
 }
 
+int cudaDeviceGetCapabilityFromC(int *value) {
+  *value = 0;
+
+  int device;
+  cudaError_t status = cudaGetDevice(&device);
+  if (status != cudaSuccess) {
+    errormessage("Error in cudaGetDevice: %s\n", "unknown error");
+    return 0;
+  }
+
+  cudaDeviceProp deviceProp;
+  status = cudaGetDeviceProperties(&deviceProp, device);
+  if (status != cudaSuccess) {
+    errormessage("Error in cudaDeviceGetProperties: %s\n", "unknown error");
+    return 0;
+  }
+
+  *value = deviceProp.major * 10 + deviceProp.minor;
+  return 1;
+}
 
 int cublasGetVersionFromC(cublasHandle_t cudaHandle, int *version) {
   cublasStatus_t status = cublasGetVersion(cudaHandle, version);
   if (status == CUBLAS_STATUS_SUCCESS) {
-    if (*version >= 111103 && *version < 120304)
+    if (*version >= 111103)
       {
       // https://docs.nvidia.com/cuda/archive//11.8.0/cuda-toolkit-release-notes/index.html
       // https://docs.nvidia.com/cuda/archive//12.3.1/cuda-toolkit-release-notes/index.html
       // Heuristics caching for the repeated CUBLAS operations was introduced in CUDA 11.8.0 and improved in CUDA 12.3.1
       // Between these versions, the caching significantly decreases the performance of cublas Gemm, Gemv
-      // so we switch the caching off for these versions
+      // More recent CUDA versions still suffer from the slowdown for the small matrices, so we switch it off completely
 #if defined(CUBLAS_VERSION) && CUBLAS_VERSION >= 111103
       cublasLtHeuristicsCacheSetCapacity(0);
 #endif
@@ -1132,4 +1152,3 @@ void cublasCaxpy_elpa_wrapper (cublasHandle_t cudaHandle, int n, float _Complex
 #ifdef __cplusplus
 }    
 #endif
-
