@@ -80,7 +80,7 @@ __global__ void gpu_copy_hvb_a_kernel(T *hvb_dev, T *a_dev, int ld_hvb, int lda,
       hvb_dev[i + ld_hvb*(ic-ics)] = a_dev[i + (l_colh-1)*lda]; // nb -> ld_hvb*(ic-ics), no compression
     }
     
-    if (my_prow == prow(ic-1, nblk, np_rows) && threadIdx.x == 0) {
+    if (my_prow == prow(ic-1, nblk, np_rows) && threadIdx.x == (l_rows-1)%blockDim.x) {
       hvb_dev[(l_rows-1) + ld_hvb*(ic-ics)] = One;
     }
     
@@ -315,9 +315,7 @@ __global__ void gpu_trmv_kernel(T *tmat_dev, T *h_dev, T *result_buffer_dev, T *
         slice_sum = elpaDeviceAdd(slice_sum, elpaDeviceMultiply(elpaDeviceComplexConjugate(tmat_dev[i + j*max_stored_rows]), h_dev[i]));
       }
 
-      // set the cache values
       cache[threadIdx.x] = slice_sum;
-      // synchronize threads in this block
       __syncthreads();
 
       // for reductions, threadsPerBlock=blockDim.x must be a power of 2
@@ -365,7 +363,6 @@ void gpu_trmv(T *tmat_dev, T *h_dev, T *result_buffer_dev, T *tau_curr_dev, int
   gpu_trmv_kernel<<<blocks,threadsPerBlock>>>            (tmat_dev, h_dev, result_buffer_dev, tau_curr_dev, max_stored_rows, n);
 #endif
   
-  // PETERDEBUG: cleanup. too much overhead, if called in tight loop?
   if (debug)
     {
     gpuDeviceSynchronize();
