@@ -48,15 +48,15 @@
 #include "config-f90.h"
 #endif
 
-#include <CL/sycl.hpp>
+#include <sycl/sycl.hpp>
 #include <stdlib.h>
 #include <stdio.h>
 
 #include <complex>
-#include <iostream>
+//#include <iostream>
 #include <cstdint>
-#include <vector>
-#include <optional>
+//#include <vector>
+//#include <optional>
 #include <type_traits>
 
 #ifndef ELPA_SYCL_IN_PROXY_APP
@@ -65,6 +65,8 @@
 #include "syclCommon.hpp"
 #endif
 
+using namespace sycl_be;
+
 // Detect complex number template arguments
 namespace {
     template <class> struct is_complex_number                    : public std::false_type {};
@@ -86,7 +88,11 @@ inline void reduction_step(T *local_mem, sycl::nd_item<1> &it) {
 
   if constexpr (wg_size >= step && sg_size < step) {
     int constexpr half_step = step >> 1;
-    local_mem[lId] += static_cast<T>(lId < half_step) * local_mem[lId + half_step];
+    if constexpr (step == wg_size) {
+      local_mem[lId] += (lId < half_step) ? local_mem[lId + half_step] : 0;
+    } else {
+      local_mem[lId] += static_cast<T>(lId < half_step) * local_mem[lId + half_step];
+    }
     it.barrier(sycl::access::fence_space::local_space);
   }
 }
@@ -160,7 +166,7 @@ __attribute__((flatten)) std::complex<T> parallel_sum_group_complex(sycl::nd_ite
 }
 
 template <typename T, int wg_size, int sg_size, bool is_using_custom_reduction=true>
-void compute_hh_trafo_c_sycl_kernel(T *q, T const *hh, T const *hh_tau, int const nev, int const nb, int const ldq, int const ncols) {
+void compute_hh_trafo_c_sycl_kernel(T *q, T const *hh, T const *hh_tau, int const nev, int const nb, int const ldq, int const ncols, QueueData *my_stream) {
   // DPC++ & SYCL 1.2.1 is gradually replaced by SYCL2020. This is to keep ELPA compatible with both old and new versions.
 #if defined(__INTEL_LLVM_COMPILER) && __INTEL_LLVM_COMPILER < 20230000
   using local_buffer = sycl::accessor<T, 1, sycl::access_mode::read_write, sycl::access::target::local>;
@@ -173,9 +179,7 @@ void compute_hh_trafo_c_sycl_kernel(T *q, T const *hh, T const *hh_tau, int cons
   #define GET_POINTER(x) x.template get_multi_ptr<sycl::access::decorated::yes>().get()
 #endif
   using sf = sycl::access::fence_space;
-  auto device = elpa::gpu::sycl::getDevice();
-  auto &queue = elpa::gpu::sycl::getQueue();
-
+  auto queue = getQueueOrDefault(my_stream);
   int constexpr q_reserve_size = wg_size;
 
   queue.submit([&](sycl::handler &h) {
@@ -186,14 +190,14 @@ void compute_hh_trafo_c_sycl_kernel(T *q, T const *hh, T const *hh_tau, int cons
     local_buffer q_s(sycl::range(nb+1), h);
     local_buffer dotp_s(sycl::range(nb+1), h);
 
-    h.parallel_for(sycl::nd_range<1>(global_range, local_range), [=](sycl::nd_item<1> it) [[intel::reqd_sub_group_size(32)]] {
+    #pragma clang diagnostic push
+    #pragma clang diagnostic ignored "-Wcuda-compat"
+    h.parallel_for(sycl::nd_range<1>(global_range, local_range), [=](sycl::nd_item<1> it) [[intel::reqd_sub_group_size(sg_size)]] {
       int tid = it.get_local_id(0);
-      int local_range = it.get_local_range(0);
 
       int j = ncols;
       int reserve_counter = q_reserve_size;
       int q_off =     it.get_group(0) + ldq * (j + tid - 1);
-      int q_off_res = it.get_group(0) + ldq * (j       - 1);
       int h_off = tid + (j - 1) * nb;
 
       q_s[tid] = q[q_off];
@@ -243,45 +247,84 @@ void compute_hh_trafo_c_sycl_kernel(T *q, T const *hh, T const *hh_tau, int cons
         }
 
         q_off -= ldq;
-        q_off_res -= ldq;
         h_off -= nb;
       }
     });
+    #pragma clang diagnostic pop
   });
   queue.wait_and_throw();
 }
 
-template <typename T>
-void launch_compute_hh_trafo_c_sycl_kernel(T *q, const T *hh, const T *hh_tau, const int nev, const int nb, const int ldq, const int ncols) {
-  int const sg_size = 32;
-  switch (nb) {
-    case 1024: compute_hh_trafo_c_sycl_kernel<T, 1024, sg_size>(q, hh, hh_tau, nev, nb, ldq, ncols); break;
-    case 512:  compute_hh_trafo_c_sycl_kernel<T, 512, sg_size>(q, hh, hh_tau, nev, nb, ldq, ncols); break;
-    case 256:  compute_hh_trafo_c_sycl_kernel<T, 256, sg_size>(q, hh, hh_tau, nev, nb, ldq, ncols); break;
-    case 128:  compute_hh_trafo_c_sycl_kernel<T, 128, sg_size>(q, hh, hh_tau, nev, nb, ldq, ncols); break;
-    case 64:   compute_hh_trafo_c_sycl_kernel<T, 64, sg_size>(q, hh, hh_tau, nev, nb, ldq, ncols); break;
-    case 32:   compute_hh_trafo_c_sycl_kernel<T, 32, sg_size>(q, hh, hh_tau, nev, nb, ldq, ncols); break;
-    case 16:   compute_hh_trafo_c_sycl_kernel<T, 16, sg_size>(q, hh, hh_tau, nev, nb, ldq, ncols); break;
-    case 8:    compute_hh_trafo_c_sycl_kernel<T, 8, sg_size>(q, hh, hh_tau, nev, nb, ldq, ncols); break;
-    case 4:    compute_hh_trafo_c_sycl_kernel<T, 4, sg_size>(q, hh, hh_tau, nev, nb, ldq, ncols); break;
-    case 2:    compute_hh_trafo_c_sycl_kernel<T, 2, sg_size>(q, hh, hh_tau, nev, nb, ldq, ncols); break;
-    case 1:    compute_hh_trafo_c_sycl_kernel<T, 1, sg_size>(q, hh, hh_tau, nev, nb, ldq, ncols); break;
-    default:   abort();
+template <typename T, int sg_size>
+void launch_compute_hh_trafo_c_sycl_kernel_sg(T *q, const T *hh, const T *hh_tau, const int nev, const int nb, const int ldq, const int ncols, QueueData *my_stream) {
+  if (nev < sg_size) {
+    switch (nb) {
+      case 1024: compute_hh_trafo_c_sycl_kernel<T, 1024, sg_size, false>(q, hh, hh_tau, nev, nb, ldq, ncols, my_stream); break;
+      case 512:  compute_hh_trafo_c_sycl_kernel<T, 512, sg_size, false>(q, hh, hh_tau, nev, nb, ldq, ncols, my_stream); break;
+      case 256:  compute_hh_trafo_c_sycl_kernel<T, 256, sg_size, false>(q, hh, hh_tau, nev, nb, ldq, ncols, my_stream); break;
+      case 128:  compute_hh_trafo_c_sycl_kernel<T, 128, sg_size, false>(q, hh, hh_tau, nev, nb, ldq, ncols, my_stream); break;
+      case 64:   compute_hh_trafo_c_sycl_kernel<T, 64, sg_size, false>(q, hh, hh_tau, nev, nb, ldq, ncols, my_stream); break;
+      case 32:   compute_hh_trafo_c_sycl_kernel<T, 32, sg_size, false>(q, hh, hh_tau, nev, nb, ldq, ncols, my_stream); break;
+      case 16:   compute_hh_trafo_c_sycl_kernel<T, 16, sg_size, false>(q, hh, hh_tau, nev, nb, ldq, ncols, my_stream); break;
+      case 8:    compute_hh_trafo_c_sycl_kernel<T, 8, sg_size, false>(q, hh, hh_tau, nev, nb, ldq, ncols, my_stream); break;
+      case 4:    compute_hh_trafo_c_sycl_kernel<T, 4, sg_size, false>(q, hh, hh_tau, nev, nb, ldq, ncols, my_stream); break;
+      case 2:    compute_hh_trafo_c_sycl_kernel<T, 2, sg_size, false>(q, hh, hh_tau, nev, nb, ldq, ncols, my_stream); break;
+      case 1:    compute_hh_trafo_c_sycl_kernel<T, 1, sg_size, false>(q, hh, hh_tau, nev, nb, ldq, ncols, my_stream); break;
+      default:   abort();
+    }
+  } else {
+    switch (nb) {
+      case 1024: compute_hh_trafo_c_sycl_kernel<T, 1024, sg_size>(q, hh, hh_tau, nev, nb, ldq, ncols, my_stream); break;
+      case 512:  compute_hh_trafo_c_sycl_kernel<T, 512, sg_size>(q, hh, hh_tau, nev, nb, ldq, ncols, my_stream); break;
+      case 256:  compute_hh_trafo_c_sycl_kernel<T, 256, sg_size>(q, hh, hh_tau, nev, nb, ldq, ncols, my_stream); break;
+      case 128:  compute_hh_trafo_c_sycl_kernel<T, 128, sg_size>(q, hh, hh_tau, nev, nb, ldq, ncols, my_stream); break;
+      case 64:   compute_hh_trafo_c_sycl_kernel<T, 64, sg_size>(q, hh, hh_tau, nev, nb, ldq, ncols, my_stream); break;
+      case 32:   compute_hh_trafo_c_sycl_kernel<T, 32, sg_size>(q, hh, hh_tau, nev, nb, ldq, ncols, my_stream); break;
+      case 16:   compute_hh_trafo_c_sycl_kernel<T, 16, sg_size>(q, hh, hh_tau, nev, nb, ldq, ncols, my_stream); break;
+      case 8:    compute_hh_trafo_c_sycl_kernel<T, 8, sg_size>(q, hh, hh_tau, nev, nb, ldq, ncols, my_stream); break;
+      case 4:    compute_hh_trafo_c_sycl_kernel<T, 4, sg_size>(q, hh, hh_tau, nev, nb, ldq, ncols, my_stream); break;
+      case 2:    compute_hh_trafo_c_sycl_kernel<T, 2, sg_size>(q, hh, hh_tau, nev, nb, ldq, ncols, my_stream); break;
+      case 1:    compute_hh_trafo_c_sycl_kernel<T, 1, sg_size>(q, hh, hh_tau, nev, nb, ldq, ncols, my_stream); break;
+      default:   abort();
+    }
   }
 }
 
-extern "C" void launch_compute_hh_trafo_c_sycl_kernel_real_double(double *q, const double *hh, const double *hh_tau, const int nev, const int nb, const int ldq, const int ncols) {
-  launch_compute_hh_trafo_c_sycl_kernel<double>(q, hh, hh_tau, nev, nb, ldq, ncols);
+template <typename T>
+void launch_compute_hh_trafo_c_sycl_kernel(T *q, const T *hh, const T *hh_tau, const int nev, const int nb, const int ldq, const int ncols, QueueData *my_stream) {
+  static int chosen_sg_size;
+
+  if (chosen_sg_size == 0) {
+    auto device = getQueueOrDefault(my_stream).get_device();
+    auto sg_sizes = device.get_info<sycl::info::device::sub_group_sizes>();
+    if (std::find(sg_sizes.begin(), sg_sizes.end(), 32) != sg_sizes.end()) chosen_sg_size = 32;
+    else if (std::find(sg_sizes.begin(), sg_sizes.end(), 64) != sg_sizes.end()) chosen_sg_size = 64;
+    else chosen_sg_size = sg_sizes[sg_sizes.size() - 1];
+  }
+  switch (chosen_sg_size) {
+    case 128: launch_compute_hh_trafo_c_sycl_kernel_sg<T, 128>(q, hh, hh_tau, nev, nb, ldq, ncols, my_stream); break;
+    case  64: launch_compute_hh_trafo_c_sycl_kernel_sg<T,  64>(q, hh, hh_tau, nev, nb, ldq, ncols, my_stream); break;
+    case  32: launch_compute_hh_trafo_c_sycl_kernel_sg<T,  32>(q, hh, hh_tau, nev, nb, ldq, ncols, my_stream); break;
+    case  16: launch_compute_hh_trafo_c_sycl_kernel_sg<T,  16>(q, hh, hh_tau, nev, nb, ldq, ncols, my_stream); break;
+    case   8: launch_compute_hh_trafo_c_sycl_kernel_sg<T,   8>(q, hh, hh_tau, nev, nb, ldq, ncols, my_stream); break;
+    case   4: launch_compute_hh_trafo_c_sycl_kernel_sg<T,   4>(q, hh, hh_tau, nev, nb, ldq, ncols, my_stream); break;
+    case   2: launch_compute_hh_trafo_c_sycl_kernel_sg<T,   2>(q, hh, hh_tau, nev, nb, ldq, ncols, my_stream); break;
+    default: abort();
+  }
 }
 
-extern "C" void launch_compute_hh_trafo_c_sycl_kernel_real_single(float *q, const float *hh, const float *hh_tau, const int nev, const int nb, const int ldq, const int ncols) {
-  launch_compute_hh_trafo_c_sycl_kernel<float>(q, hh, hh_tau, nev, nb, ldq, ncols);
+extern "C" void launch_compute_hh_trafo_c_sycl_kernel_real_double(double *q, const double *hh, const double *hh_tau, const int nev, const int nb, const int ldq, const int ncols, QueueData *my_stream) {
+  launch_compute_hh_trafo_c_sycl_kernel<double>(q, hh, hh_tau, nev, nb, ldq, ncols, my_stream);
 }
 
-extern "C" void launch_compute_hh_trafo_c_sycl_kernel_complex_double(std::complex<double> *q, const std::complex<double> *hh, const std::complex<double> *hh_tau, const int nev, const int nb, const int ldq, const int ncols) {
-  launch_compute_hh_trafo_c_sycl_kernel<std::complex<double>>(q, hh, hh_tau, nev, nb, ldq, ncols);
+extern "C" void launch_compute_hh_trafo_c_sycl_kernel_real_single(float *q, const float *hh, const float *hh_tau, const int nev, const int nb, const int ldq, const int ncols, QueueData *my_stream) {
+  launch_compute_hh_trafo_c_sycl_kernel<float>(q, hh, hh_tau, nev, nb, ldq, ncols, my_stream);
 }
 
-extern "C" void launch_compute_hh_trafo_c_sycl_kernel_complex_single(std::complex<float> *q, const std::complex<float> *hh, const std::complex<float> *hh_tau, const int nev, const int nb, const int ldq, const int ncols) {
-  launch_compute_hh_trafo_c_sycl_kernel<std::complex<float>>(q, hh, hh_tau, nev, nb, ldq, ncols);
+extern "C" void launch_compute_hh_trafo_c_sycl_kernel_complex_double(std::complex<double> *q, const std::complex<double> *hh, const std::complex<double> *hh_tau, const int nev, const int nb, const int ldq, const int ncols, QueueData *my_stream) {
+  launch_compute_hh_trafo_c_sycl_kernel<std::complex<double>>(q, hh, hh_tau, nev, nb, ldq, ncols, my_stream);
+}
+
+extern "C" void launch_compute_hh_trafo_c_sycl_kernel_complex_single(std::complex<float> *q, const std::complex<float> *hh, const std::complex<float> *hh_tau, const int nev, const int nb, const int ldq, const int ncols, QueueData *my_stream) {
+  launch_compute_hh_trafo_c_sycl_kernel<std::complex<float>>(q, hh, hh_tau, nev, nb, ldq, ncols, my_stream);
 }
