@@ -271,6 +271,7 @@ static const elpa_index_int_entry_t int_entries[] = {
         BOOL_ENTRY("cannon_for_generalized", "Whether to use Cannons algorithm for the generalized EVP" , 0, ELPA_AUTOTUNE_NOT_TUNABLE, ELPA_AUTOTUNE_NOT_TUNABLE, 0, ELPA_AUTOTUNE_PART_NONE, PRINT_YES),
         BOOL_ENTRY("pxtrmm_for_generalized", "Whether to use ScaLAPACK's PxTRMM for the generalized EVP", 0, ELPA_AUTOTUNE_NOT_TUNABLE, ELPA_AUTOTUNE_NOT_TUNABLE, 0, ELPA_AUTOTUNE_PART_NONE, PRINT_YES),
         BOOL_ENTRY("pxgemm_for_generalized", "Whether to use elpa_pxgemm_multiply for the generalized EVP", 0, ELPA_AUTOTUNE_NOT_TUNABLE, ELPA_AUTOTUNE_NOT_TUNABLE, 0, ELPA_AUTOTUNE_PART_NONE, PRINT_YES),
+        BOOL_ENTRY("multiply_at_a", "Switch to A^T * A in the matrix multiplication routine",  0, ELPA_AUTOTUNE_NOT_TUNABLE, ELPA_AUTOTUNE_NOT_TUNABLE, 0, ELPA_AUTOTUNE_PART_NONE, PRINT_YES),
 #if defined(THREADING_SUPPORT_CHECK) && defined(ALLOW_THREAD_LIMITING) && !defined(HAVE_SUFFICIENT_MPI_THREADING_SUPPORT)
         BOOL_ENTRY("limit_openmp_threads", "Limit the number if openmp threads to 1", 0, ELPA_AUTOTUNE_NOT_TUNABLE, ELPA_AUTOTUNE_NOT_TUNABLE, 0, ELPA_AUTOTUNE_PART_NONE, PRINT_NO),
 #endif
@@ -278,10 +279,10 @@ static const elpa_index_int_entry_t int_entries[] = {
         INT_ENTRY("cannon_buffer_size", "Increasing the buffer size might make it faster, but costs memory", 0, ELPA_AUTOTUNE_NOT_TUNABLE, ELPA_AUTOTUNE_NOT_TUNABLE, ELPA_AUTOTUNE_DOMAIN_ANY,  ELPA_AUTOTUNE_PART_NONE, \
                         cannon_buffer_size_cardinality, cannon_buffer_size_enumerate, cannon_buffer_size_is_valid, NULL, PRINT_YES),
         // tunables
-#if defined(WITH_NVIDIA_NCCL) || defined(WITH_AMD_RCCL)
-        BOOL_ENTRY("use_ccl", "Use NVIDIA's nccl or AMD's rccl communication libraries", 1, ELPA_AUTOTUNE_GPU, ELPA_AUTOTUNE_GPU, ELPA_AUTOTUNE_DOMAIN_ANY, ELPA_AUTOTUNE_PART_ANY, PRINT_YES), 
+#if defined(WITH_NVIDIA_NCCL) || defined(WITH_AMD_RCCL) || defined(WITH_ONEAPI_ONECCL)
+        BOOL_ENTRY("use_ccl", "Use NVIDIA's nccl, AMD's rccl or Intel's oneCCL communication libraries", 1, ELPA_AUTOTUNE_GPU, ELPA_AUTOTUNE_GPU, ELPA_AUTOTUNE_DOMAIN_ANY, ELPA_AUTOTUNE_PART_ANY, PRINT_YES), 
 #else
-        BOOL_ENTRY("use_ccl", "Use NVIDIA's nccl or AMD's rccl communication libraries", 0, ELPA_AUTOTUNE_GPU, ELPA_AUTOTUNE_GPU, ELPA_AUTOTUNE_DOMAIN_ANY, ELPA_AUTOTUNE_PART_ANY, PRINT_YES), 
+        BOOL_ENTRY("use_ccl", "Use NVIDIA's nccl, AMD's rccl or Intel's oneCCL communication libraries", 0, ELPA_AUTOTUNE_GPU, ELPA_AUTOTUNE_GPU, ELPA_AUTOTUNE_DOMAIN_ANY, ELPA_AUTOTUNE_PART_ANY, PRINT_YES), 
 #endif
 	// 1. non-blocking MPI
         INT_ENTRY("nbc_row_global_gather", "Use non blocking collectives for rows in global_gather", 0, ELPA_AUTOTUNE_NOT_TUNABLE, ELPA_AUTOTUNE_SOLVE, ELPA_AUTOTUNE_DOMAIN_ANY, ELPA_AUTOTUNE_PART_ANY, \
