@@ -120,9 +120,6 @@
 #undef  gpu_copy_PRECISION_tmp2_tmat2
 #undef  gpu_copy_PRECISION_a_tmat1
 #undef  gpu_copy_PRECISION_a_tmatc
-#undef  gpu_copy_PRECISION_a_aux_bc
-#undef  gpu_copy_PRECISION_aux_bc_aux_mat
-#undef  gpu_copy_PRECISION_tmp2_c
 #undef  gpu_copy_and_set_zeros_PRECISION
 #undef  gpu_dot_product_PRECISION
 #undef  gpu_dot_product_and_assign_PRECISION
@@ -157,23 +154,7 @@
 #undef  GPU_GET_SKEWSYMMETRIC_SECOND_HALF_Q_PRECISION
 #undef  GPU_PUT_SKEWSYMMETRIC_SECOND_HALF_Q_PRECISION
 
-#undef GPU_DISTRIBUTE_GLOBAL_COLUMN_PRECISION
-#undef GPU_COPY_QMAT1_TO_QMAT2_PRECISION
-#undef GPU_UPDATE_D_PRECISION
-#undef GPU_COPY_D_TO_D_TMP_PRECISION
-
-#undef GPU_COPY_Q_SLICE_TO_QTMP1_PRECISION
-#undef GPU_ZERO_Q_PRECISION
-#undef GPU_COPY_QTMP1_TO_QTMP1_TMP_PRECISION
-#undef GPU_FILL_TMP_ARRAYS_PRECISION
-#undef GPU_COPY_QTMP1_SLICE_TO_Q_PRECISION
-#undef GPU_COPY_Q_SLICE_TO_QTMP2_PRECISION
-#undef GPU_FILL_EV_PRECISION
-#undef GPU_COPY_QTMP2_SLICE_TO_Q_PRECISION
-
 #undef  gpusolver_PRECISION_syevd 
-#undef GPU_CONSTRUCT_TRIDI_MATRIX_PRECISION
-#undef GPU_CHECK_MONOTONY_PRECISION
 #if 0
 /* General definitions needed in single and double case */
 /* the if 0 bracket is just to make the IBM Fortran compiler happy */
@@ -248,9 +229,6 @@
 #define  gpu_copy_PRECISION_tmp2_tmat2 gpu_copy_double_tmp2_tmat2
 #define  gpu_copy_PRECISION_a_tmat1 gpu_copy_double_a_tmat1
 #define  gpu_copy_PRECISION_a_tmatc gpu_copy_double_a_tmatc
-#define  gpu_copy_PRECISION_a_aux_bc gpu_copy_double_a_aux_bc
-#define  gpu_copy_PRECISION_aux_bc_aux_mat gpu_copy_double_aux_bc_aux_mat
-#define  gpu_copy_PRECISION_tmp2_c gpu_copy_double_tmp2_c
 #define  gpu_copy_and_set_zeros_PRECISION gpu_copy_and_set_zeros_double
 #define  gpu_dot_product_PRECISION gpu_dot_product_double
 #define  gpu_dot_product_and_assign_PRECISION gpu_dot_product_and_assign_double
@@ -287,24 +265,7 @@
 #define GPU_GET_SKEWSYMMETRIC_SECOND_HALF_Q_PRECISION gpu_get_skewsymmetric_second_half_q_double
 #define GPU_PUT_SKEWSYMMETRIC_SECOND_HALF_Q_PRECISION gpu_put_skewsymmetric_second_half_q_double
 
-#define GPU_DISTRIBUTE_GLOBAL_COLUMN_PRECISION gpu_distribute_global_column_double
-#define GPU_COPY_QMAT1_TO_QMAT2_PRECISION gpu_copy_qmat1_to_qmat2_double
-#define GPU_UPDATE_D_PRECISION gpu_update_d_double
-#define GPU_COPY_D_TO_D_TMP_PRECISION gpu_copy_d_to_d_tmp_double
-
-#define GPU_COPY_Q_SLICE_TO_QTMP1_PRECISION gpu_copy_q_slice_to_qtmp1_double
-#define GPU_ZERO_Q_PRECISION gpu_zero_q_double
-#define GPU_COPY_QTMP1_TO_QTMP1_TMP_PRECISION gpu_copy_qtmp1_to_qtmp1_tmp_double
-#define GPU_FILL_TMP_ARRAYS_PRECISION gpu_fill_tmp_arrays_double
-#define GPU_COPY_QTMP1_SLICE_TO_Q_PRECISION gpu_copy_qtmp1_slice_to_q_double
-#define GPU_COPY_Q_SLICE_TO_QTMP2_PRECISION gpu_copy_q_slice_to_qtmp2_double
-#define GPU_FILL_EV_PRECISION gpu_fill_ev_double
-#define GPU_COPY_QTMP2_SLICE_TO_Q_PRECISION gpu_copy_qtmp2_slice_to_q_double
-
 #define  gpusolver_PRECISION_syevd gpusolver_dsyevd
-#define GPU_CONSTRUCT_TRIDI_MATRIX_PRECISION gpu_construct_tridi_matrix_double
-
-#define GPU_CHECK_MONOTONY_PRECISION gpu_check_monotony_double
 #endif /* DOUBLE_PRECISION */
 
 #ifdef SINGLE_PRECISION
@@ -375,9 +336,6 @@
 #define  gpu_copy_PRECISION_tmp2_tmat2 gpu_copy_float_tmp2_tmat2
 #define  gpu_copy_PRECISION_a_tmat1 gpu_copy_float_a_tmat1
 #define  gpu_copy_PRECISION_a_tmatc gpu_copy_float_a_tmatc
-#define  gpu_copy_PRECISION_a_aux_bc gpu_copy_float_a_aux_bc
-#define  gpu_copy_PRECISION_aux_bc_aux_mat gpu_copy_float_aux_bc_aux_mat
-#define  gpu_copy_PRECISION_tmp2_c gpu_copy_float_tmp2_c
 #define  gpu_copy_and_set_zeros_PRECISION gpu_copy_and_set_zeros_float
 #define  gpu_dot_product_PRECISION gpu_dot_product_float
 #define  gpu_dot_product_and_assign_PRECISION gpu_dot_product_and_assign_float
@@ -414,24 +372,7 @@
 #define GPU_GET_SKEWSYMMETRIC_SECOND_HALF_Q_PRECISION gpu_get_skewsymmetric_second_half_q_float
 #define GPU_PUT_SKEWSYMMETRIC_SECOND_HALF_Q_PRECISION gpu_put_skewsymmetric_second_half_q_float
 
-#define GPU_DISTRIBUTE_GLOBAL_COLUMN_PRECISION gpu_distribute_global_column_float
-#define GPU_COPY_QMAT1_TO_QMAT2_PRECISION gpu_copy_qmat1_to_qmat2_float
-#define GPU_UPDATE_D_PRECISION gpu_update_d_float
-#define GPU_COPY_D_TO_D_TMP_PRECISION gpu_copy_d_to_d_tmp_float
-
-#define GPU_COPY_Q_SLICE_TO_QTMP1_PRECISION gpu_copy_q_slice_to_qtmp1_float
-#define GPU_ZERO_Q_PRECISION gpu_zero_q_float
-#define GPU_COPY_QTMP1_TO_QTMP1_TMP_PRECISION gpu_copy_qtmp1_to_qtmp1_tmp_float
-#define GPU_FILL_TMP_ARRAYS_PRECISION gpu_fill_tmp_arrays_float
-#define GPU_COPY_QTMP1_SLICE_TO_Q_PRECISION gpu_copy_qtmp1_slice_to_q_float
-#define GPU_COPY_Q_SLICE_TO_QTMP2_PRECISION gpu_copy_q_slice_to_qtmp2_float
-#define GPU_FILL_EV_PRECISION gpu_fill_ev_float
-#define GPU_COPY_QTMP2_SLICE_TO_Q_PRECISION gpu_copy_qtmp2_slice_to_q_float
-
-#define  gpusolver_PRECISION_syevd gpusolver_ssyevd
-#define GPU_CONSTRUCT_TRIDI_MATRIX_PRECISION gpu_construct_tridi_matrix_float
-
-#define GPU_CHECK_MONOTONY_PRECISION gpu_check_monotony_float
+#define gpusolver_PRECISION_syevd gpusolver_ssyevd
 #endif /* SINGLE_PRECISION */
 
 #endif /* REALCASE */
@@ -514,9 +455,6 @@
 #undef  gpu_copy_PRECISION_tmp2_tmat2
 #undef  gpu_copy_PRECISION_a_tmat1
 #undef  gpu_copy_PRECISION_a_tmatc
-#undef  gpu_copy_PRECISION_a_aux_bc
-#undef  gpu_copy_PRECISION_aux_bc_aux_mat
-#undef  gpu_copy_PRECISION_tmp2_c
 #undef  gpu_copy_and_set_zeros_PRECISION
 #undef  gpu_dot_product_PRECISION
 #undef  gpu_dot_product_and_assign_PRECISION
@@ -634,9 +572,6 @@
 #define  gpu_copy_PRECISION_tmp2_tmat2 gpu_copy_double_complex_tmp2_tmat2
 #define  gpu_copy_PRECISION_a_tmat1 gpu_copy_double_complex_a_tmat1
 #define  gpu_copy_PRECISION_a_tmatc gpu_copy_double_complex_a_tmatc
-#define  gpu_copy_PRECISION_a_aux_bc gpu_copy_double_complex_a_aux_bc
-#define  gpu_copy_PRECISION_aux_bc_aux_mat gpu_copy_double_complex_aux_bc_aux_mat
-#define  gpu_copy_PRECISION_tmp2_c gpu_copy_double_complex_tmp2_c
 #define  gpu_copy_and_set_zeros_PRECISION gpu_copy_and_set_zeros_double_complex
 #define  gpu_dot_product_PRECISION gpu_dot_product_double_complex
 #define  gpu_dot_product_and_assign_PRECISION gpu_dot_product_and_assign_double_complex
@@ -744,9 +679,6 @@
 #define  gpu_copy_PRECISION_tmp2_tmat2 gpu_copy_float_complex_tmp2_tmat2
 #define  gpu_copy_PRECISION_a_tmat1 gpu_copy_float_complex_a_tmat1
 #define  gpu_copy_PRECISION_a_tmatc gpu_copy_float_complex_a_tmatc
-#define  gpu_copy_PRECISION_a_aux_bc gpu_copy_float_complex_a_aux_bc
-#define  gpu_copy_PRECISION_aux_bc_aux_mat gpu_copy_float_complex_aux_bc_aux_mat
-#define  gpu_copy_PRECISION_tmp2_c gpu_copy_float_complex_tmp2_c
 #define  gpu_copy_and_set_zeros_PRECISION gpu_copy_and_set_zeros_float_complex
 #define  gpu_dot_product_PRECISION gpu_dot_product_float_complex
 #define  gpu_dot_product_and_assign_PRECISION gpu_dot_product_and_assign_float_complex
