@@ -505,6 +505,9 @@ int main(int argc, char** argv) {
 #if TEST_INTEL_GPU == 1 || TEST_INTEL_GPU_OPENMP == 1  || TEST_INTEL_GPU_SYCL == 1
   elpa_set(handle, "intel-gpu", TEST_GPU, &error_elpa);
   assert_elpa_ok(error_elpa);
+
+  elpa_set(handle, "sycl_show_all_devices", 0, &error_elpa);
+  assert_elpa_ok(error_elpa);
 #endif
 
 #if defined(TEST_SOLVE_2STAGE) && defined(TEST_KERNEL)
@@ -553,12 +556,27 @@ int main(int argc, char** argv) {
   assert_elpa_ok(error_elpa);
 #endif /* defined(TEST_SOLVE_2STAGE) && defined(TEST_KERNEL) */
 
-#if (TEST_GPU_SET_ID == 1) && (TEST_INTEL_GPU == 0) && (TEST_INTEL_GPU_OPENMP == 0) && (TEST_INTEL_GPU_SYCL == 0)
+
+#if TEST_GPU_SET_ID == 1 && (TEST_INTEL_GPU == 0) && (TEST_INTEL_GPU_OPENMP == 0) && (TEST_INTEL_GPU_SYCL == 0)
+#ifdef DEBUG_SYCL_ON_CPU
+// for SYCL on CPU case: gpu_id and device_pointer_api tests don't make sense and are disabled
+#ifdef WITH_MPI
+  MPI_Finalize();
+#endif
+  exit(77);
+#endif /* DEBUG_SYCL_ON_CPU */
+
   int numberOfDevices;
   gpuGetDeviceCount_tests(&numberOfDevices);
   printf("Number of Devices found: %d\n\n", numberOfDevices);
+
+  if (numberOfDevices==0) {
+    if (myid==0) printf("No GPU devices found! Aborting...\n");
+    exit(1);
+  }
+
   gpuID = myid%numberOfDevices;
-  printf("gpuID: %i\n", gpuID);
+
   elpa_set(handle, "use_gpu_id", gpuID, &error_elpa);
   assert_elpa_ok(error_elpa);
 #endif
@@ -570,21 +588,14 @@ int main(int argc, char** argv) {
 #if TEST_GPU_DEVICE_POINTER_API == 1
   set_gpu_parameters_tests();
 
-#if TEST_INTEL_GPU_SYCL == 1 /* temporary fix for SYCL on CPU */
-  int numberOfDevices=0;
-  successGPU = syclGetCpuCount(numberOfDevices);
-  if (!successGPU){
-    printf("Error in syclGetCpuCount\n");
-    exit(1);
-  }
-#endif
-
   // Set device
+#if (TEST_INTEL_GPU == 0) && (TEST_INTEL_GPU_OPENMP == 0) && (TEST_INTEL_GPU_SYCL == 0)
   successGPU = gpuSetDevice_tests(gpuID);
   if (!successGPU){
     printf("Error in gpuSetDevice\n");
     exit(1);
   }
+#endif
 
 #if defined(TEST_EIGENVECTORS)
   // malloc
@@ -676,9 +687,6 @@ int main(int argc, char** argv) {
 #endif /* defined(TEST_GENERALIZED_EIGENPROBLEM) && defined(TEST_MATRIX_RANDOM) */
 
 #if defined(TEST_CHOLESKY)
-  elpa_set(handle, "gpu_cholesky", 1, &error_elpa);
-  assert_elpa_ok(error_elpa);
-
   successGPU = gpuMalloc_tests((intptr_t *) &a_dev , na_rows*na_cols*sizeof(MATRIX_TYPE));
   if (!successGPU){
     fprintf(stderr, "Error in gpuMalloc(a_dev)\n");
@@ -695,9 +703,6 @@ int main(int argc, char** argv) {
 #endif /* TEST_CHOLESKY */
 
 #if defined(TEST_MULTIPLY)
-  elpa_set(handle, "gpu_hermitian_multiply", 1, &error_elpa);
-  assert_elpa_ok(error_elpa);
-
   successGPU = gpuMalloc_tests((intptr_t *) &a_dev , na_rows*na_cols*sizeof(MATRIX_TYPE));
   if (!successGPU){
     fprintf(stderr, "Error in gpuMalloc(a_dev)\n");
